#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2013 Virantha Ekanayake All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import argparse
import sys, os
import logging
import glob
import shutil


from version import __version__
from flickr import Flickr
from flashair import FlashAir

class AirFrame(object):

    def __init__(self):
        pass

    def _parse_csv_list(self, s):
        try:
            assert isinstance(s,str)
            s = s.strip()
            value_list = s.split(',')
        except:
            raise argparse.ArgumentTypeError("Could not parse tag list")
        return value_list

    def get_options(self, argv):
        """
            Parse the command-line options and set the following object properties:

            I really need to convert this to use docopt!

            :param argv: usually just sys.argv[1:]
            :returns: Nothing

            :ivar debug: Enable logging debug statements
            :ivar verbose: Enable verbose logging
        """
        p = argparse.ArgumentParser(
                description = "Push pictures from Flickr or local files to a Toshiba FlashAir automatically",
                epilog = "AirFrame version %s (Copyright 2014 Virantha Ekanayake)" % __version__,
                )

        p.add_argument('-l', '--local-dir', type=str,
            help='Upload all .jpg files from this directory instead of Flickr')

        p.add_argument('-d', '--debug', action='store_true',
            default=False, dest='debug', help='Turn on debugging')

        p.add_argument('-v', '--verbose', action='store_true',
            default=False, dest='verbose', help='Turn on verbose mode')

        p.add_argument('-f', '--force', action='store_true',
            default=False, help='Force upload of all pictures to Flashair (instead of only new pictures)')

        p.add_argument('-n', '--number', type=int,
            default=100, dest='number', help='Max number of photos to sync')

        p.add_argument('-t', '--tags', type=self._parse_csv_list,
                default=[], dest='tags', help='List of tags to match')

        p.add_argument('flashair_ip', type=str,
                        help='The ip/hostname of your FlashAir card')


        args = p.parse_args(argv)

        self.debug = args.debug
        self.verbose = args.verbose
        self.photo_count = args.number
        self.photo_tags = args.tags
        self.force_upload = args.force
        self.flashair_ip = args.flashair_ip
        self.local_dir = args.local_dir

        if self.debug:
            logging.basicConfig(level=logging.DEBUG, format='%(message)s')

        if self.verbose:
            logging.basicConfig(level=logging.INFO, format='%(message)s')

    def flickr_mode(self):
        # Connect to Flickr
        logging.debug("list of tags: %s" % self.photo_tags)
        self.flickr = Flickr()
        if len(self.photo_tags) > 0:
            photo_filenames = self.flickr.get_tagged(self.photo_tags, self.photo_count, download_dir=self.download_dir)
        else:
            photo_filenames = self.flickr.get_recent(self.photo_count,download_dir=self.download_dir)
        return photo_filenames

    def local_dir_mode(self):
        # Copy all the files in the named directory to the cache (download_dir).
        # completely replaces both the local cached files and the ones on the 
        # wifi sd card
        pattern = '/*.[jJ][pP][gG]'
        match = self.local_dir + pattern
        
        if os.path.isdir(self.download_dir):
            logging.debug("removing existing cache dir: %s", self.download_dir)
            shutil.rmtree(self.download_dir)
        
        os.mkdir(self.download_dir)
        logging.debug("caching files from: %s" % match)
        photo_filenames = glob.glob(match)
        for filename in photo_filenames:
            logging.debug("copy %s to %s" % (filename, self.download_dir))
            shutil.copy(filename, self.download_dir)
        return photo_filenames    

    def go(self, argv):
        self.download_dir = ".airframe"
        self.get_options(argv)

        if self.local_dir:
            photo_filenames = self.local_dir_mode()
        else:
            photo_filenames = self.flickr_mode()

        self.flashair = FlashAir(self.flashair_ip)
        self.flashair.sync_files_on_card_to_list(photo_filenames, self.force_upload)

def main():
    script = AirFrame()
    script.go(sys.argv[1:])

if __name__ == '__main__':
    main()





#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import io
import sys


from setuptools import setup, find_packages

import airframe
from airframe.version import __version__

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)

long_description = read('README.rst', 'HISTORY.rst', 'TODO.rst')

packages = find_packages(exclude="tests")

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name='airframe',
    version=__version__,
    description='Push images to a Toshiba FlashAir Wifi SD card',
    long_description=long_description,
    author='Virantha N. Ekanayake',
    author_email='virantha@gmail.com',
    url='https://github.com/virantha/airframe',
    packages=packages,
    include_package_data=True,
    install_requires=required,
    license="ASL2",
    zip_safe=True,
    keywords='airframe',
    entry_points = {
            'console_scripts': [
                    'airframe = airframe.airframe:main'
                ],
        },
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
    ],
    test_suite='tests',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2013 Virantha Ekanayake All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import argparse
import sys, os
import logging

import yaml
import flickrapi
import urllib


class Photo(object):
    def __init__(self, photo_element):
        """Construct a photo object out of the XML response from Flickr"""
        attrs = { 'farm': 'farmid', 'server':'serverid','id':'photoid','secret':'secret'}
        for flickr_attr, py_attr in attrs.items():
            setattr(self, py_attr, photo_element.get(flickr_attr))
        
    def _construct_flickr_url(self):
        url = "http://farm%s.staticflickr.com/%s/%s_%s_b.jpg" % (self.farmid,self.serverid, self.photoid, self.secret)
        return url

    def download_photo(self, dirname, cache=False, tgt_filename=None):
        if not os.path.exists(dirname):
            os.makedirs(dirname)
        tgt = os.path.join(dirname, "%s.jpg" % self.photoid)
        if cache:
            if os.path.isfile(tgt):
                return tgt
        urllib.urlretrieve(self._construct_flickr_url(), tgt)
        return tgt
        
class Flickr(object):

    def __init__(self):
        self.set_keys(*self.read_keys())
        self.get_auth2()

    def read_keys(self):
        """
            Read the flickr API key and secret from a local file
        """
        with open("flickr_api.yaml") as f:
            api = yaml.load(f)
        return (api["key"], api["secret"])

    def set_keys(self, key, secret):
        self.api_key = key
        self.api_secret = secret

    def get_auth2(self):
        print("Authenticating to Flickr")
        self.flickr = flickrapi.FlickrAPI(self.api_key, self.api_secret)
        (token,frob) = self.flickr.get_token_part_one(perms='read')
        if not token: raw_input("Press ENTER after you authorized this program")
        self.flickr.get_token_part_two((token,frob))
        print("Authentication succeeded")


    def get_tagged(self, tags, count, download_dir="photos"):
        """ Get photos with the given list of tags
        """
        print ("connecting to flickr, and getting %d photos with tags %s" % (count, tags))
        x = self.flickr.photos_search(api_key = self.api_key, user_id="me", tags=','.join(tags), per_page=count)
        photos = self._extract_photos_from_xml(x)
        photo_filenames = self._sync_photos(photos, download_dir)
        print("Found %d photos" % len(photos))
        return photo_filenames


    def _sync_photos(self, photos, download_dir="photos", clean_up=False):
        """
            Connect to flickr, and for each photo in the list, download.
            Then, if delete photos that are present locally that weren't present in the list of photos.

            :returns: List of filenames downloaded
        """
        photo_filenames = []
        photo_count = len(photos)
        for i,photo in enumerate(photos):
            print("[%d/%d] Downloading %s from flickr" % (i,photo_count,photo.photoid))
            filename = photo.download_photo(download_dir, cache=True)
            photo_filenames.append(filename)

        # Now, go through and clean up directory if required
        
        if clean_up:
            photo_file_list = ["%s.jpg" % (x.photoid) for x in photos]
            for fn in os.listdir(download_dir):
                full_fn = os.path.join(download_dir, fn)
                if os.path.isfile(full_fn):
                    if not fn in photo_file_list:
                        print ("Flickr sync: Deleting file %s" % fn)
                        os.remove(full_fn)

        return photo_filenames

    def _extract_photos_from_xml(self, xml):
        photos = []
        for i in xml.iter():
            if i.tag == 'rsp':
                # the response header.  stat member should be 'ok'
                if i.get('stat') == 'ok':
                    continue
                else:
                    # error, so just break
                    break
            if i.tag == 'photo':
                photos.append(Photo(i))
        return photos

    def get_recent(self,count, download_dir="photos"):
        """ get the most recent photos
        """
        print ("connecting to flickr, and getting most recent %d photos" % count)
        x = self.flickr.people_getphotos(api_key = self.api_key, user_id="me",per_page=count)
        #x = self.flickr.photos_search(api_key=self.api_key,"me")

        photos = self._extract_photos_from_xml(x)
        photo_filenames = self._sync_photos(photos, download_dir)
        return photo_filenames


def main():
    #logging.basicConfig(level=logging.DEBUG, format='%(message)s')
    script = Flickr()
    key,secret = script.read_keys()
    script.set_keys(key,secret)
    script.get_auth2()
    script.get_recent(10)


if __name__ == '__main__':
    main()





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2013 Virantha Ekanayake All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys, os, shutil
import logging
import requests

import hashlib
import time

class FlashAir(object):
    """
        Interface to the REST API of the Toshiba FlashAir card.  
        See the documentation to their API at:
            https://flashair-developers.com/en/
    """
    def __init__(self, hostname):
        """ 
            :param hostname: IP address or hostname of the FlashAir card
            :type hostname: string
        """
        self.hostname = hostname
        self.card_path = "/DCIM/100__TSB"

    def delete_file(self, filename):
        payload = {"op":100, "DIR":self.card_path}
        r = requests.get("http://%s/command.cgi" % self.hostname, params=payload)

    def get_file_list(self):
        """

            The SD card returns a list that looks like the following:

            WLANSD_FILELIST
            /DCIM/100__TSB,FA000001.JPG,128751,33,16602,18432
            /DCIM/100__TSB,FLASH3.JPG,370952,32,0,0
            /DCIM/100__TSB,FLASH5.JPG,287034,32,0,0
            /DCIM/100__TSB,FLASH6.JPG,387163,32,17295,31508
            /DCIM/100__TSB,FLASH7.JPG,533998,32,17295,31520

        """
        payload = {"op":100, "DIR":self.card_path}
        r = requests.get("http://%s/command.cgi" % self.hostname, params=payload)
        r.raise_for_status()
        print r.text
        # Divide the returned text by newline, and ignore the first line "WLANSD_FILELIST"
        lines = r.text.split('\n')
        assert lines[0].strip()=='WLANSD_FILELIST'

        filenames = []
        for line in lines[1:]:
            # Split each line by ',' and take the second column for the filename
            values = line.split(',')
            if len(values) > 2:
                filename = values[1].strip()
                filenames.append(filename)

        print filenames
        return filenames

    def _get_renamed_filename(self, filename):
        # First, separate out the path from the filename
        path = os.path.dirname(filename)
        fn = os.path.basename(filename)
        h = hashlib.sha1()
        h.update(fn)
        hash_filename = "%s.JPG" % (h.hexdigest()[:8].upper())

        hash_full_filename = os.path.join(path, hash_filename)
        logging.debug("Hashed file: %s" % (hash_full_filename))
        return hash_full_filename


    def copy_and_rename_file(self, filename):
        """
            :returns: hashed filename in 8.3 format with JPG extension
        """
        hash_full_filename = self._get_renamed_filename(filename)

        # First, separate out the path from the filename
        path = os.path.dirname(filename)
        fn = os.path.basename(filename)
        hash_filename = os.path.basename(hash_full_filename)

        # Rename the file to the hashed filename
        cwd = os.getcwd()
        os.chdir(path)
        if not os.path.exists(hash_filename):
            shutil.copy(fn, hash_filename)
            logging.debug("Copied %s to %s" % (fn, hash_filename))
        os.chdir(cwd)
        return hash_full_filename

    def delete_file(self, filename):
        # Change the upload directory on the card
        url = "http://%s/upload.cgi" % self.hostname

        payload = {'DEL': "%s/%s" % (self.card_path, filename)}
        r = requests.get(url, params=payload)
        r.raise_for_status()

    def _set_write_protect(self):
        # Change the upload directory on the card
        url = "http://%s/upload.cgi" % self.hostname

        payload = {'WRITEPROTECT': "ON"}
        r = requests.get(url, params=payload)
        r.raise_for_status()
        if not "SUCCESS" in r.content:
            print("Could not put card into host-write-protect mode")


    def set_timestamp(self, t):
        """
            :param t: The time stamp
            :type t: time.struct_time
        """
        fat32_time = ((t.tm_year-1980)<<25) | (t.tm_mon << 21) | (t.tm_mday << 16) | (t.tm_hour << 11) | (t.tm_min << 5) | (t.tm_sec >>1)

        #print("Setting timestamp to %d (0x%0.8X)" % (fat32_time, fat32_time))
        url = "http://%s/upload.cgi" % self.hostname
        payload = {'FTIME': "0x%0.8X" % fat32_time}
        r = requests.get(url, params=payload)
        r.raise_for_status()

    def upload_file(self, filename):
        # First, set the time of the photo to right now
        self.set_timestamp(time.localtime())
        # Change the upload directory on the card
        url = "http://%s/upload.cgi" % self.hostname

        payload = {'UPDIR':self.card_path}
        r = requests.get(url, params=payload)
        r.raise_for_status()

        # Now, upload the file
        hash_full_filename = self._get_renamed_filename(filename)
        hash_filename = os.path.basename(hash_full_filename)

        files = {'file':(hash_filename, open(filename,'rb'))}
        r = requests.post(url, files=files)
        r.raise_for_status()
        
    def sync_files_on_card_to_list(self, filename_list, force=False):
        """
            Delete any files on the SD card that are not present in the list.
            Upload any files not already present on the card (based on name).
            Ignore any files already present on the card.
        """

        self._set_write_protect()
        # First, get the file list on the card
        sd_file_list = self.get_file_list()
        
        hashed_local_list = [os.path.basename(self._get_renamed_filename(x)) for x in filename_list]

        # If force upload, we are going to delete all the files in this directory
        if force:
            files_to_delete_list = sd_file_list
        else:
            # Delete any file not present in the filename_list
            files_to_delete_list = [ fn for fn in sd_file_list if not fn in hashed_local_list]

        #for fn in sd_file_list:
            #if not fn in hashed_local_list:
        for fn in files_to_delete_list:
            print("Deleting file %s on FlashAir" % fn)
            self.delete_file(fn)

        # Now, for any file not already present in the SD card, upload it
        i = 0
        n = len(filename_list)
        for fn, hash_fn in zip(filename_list, hashed_local_list):

            i+=1
            if force or not hash_fn in sd_file_list:
                print("[%d/%d] Uploading file %s to %s on FlashAir" % (i,n,fn, hash_fn))
                self.upload_file(fn)

            else:
                print("[%d/%d] Uploading file %s to %s on FlashAir: SKIPPED(already present)" % (i,n,fn, hash_fn))

def main():
    #logging.basicConfig(level=logging.DEBUG, format='%(message)s')
    script = FlashAir("192.168.9.70")
    file_list = script.get_file_list()
    #newfile = script.copy_and_rename_file("photos/11709952505.jpg")
    file_list =[ "photos/11692277333.jpg",
                "photos/11692418594.jpg",
                "photos/11692766176.jpg",
                "photos/11692787166.jpg",
                "photos/11699467984.jpg",
                "photos/11699860336.jpg",
                "photos/11709836005.jpg",
                "photos/11709952505.jpg",
                "photos/11710078243.jpg",
                "photos/11710080573.jpg",
                "photos/11710081763.jpg",]
    #script.upload_file("photos/11709836005.jpg")
    script.sync_files_on_card_to_list(file_list)
    

if __name__ == '__main__':
    main()


